@@ -195,6 +195,15 @@ extern "C" {
     case OFTString:
       PROTECT(ans=allocVector(STRSXP,nRows));
       break;
+      case OFTDate:
+	PROTECT(ans=allocVector(STRSXP,nRows));
+	break;
+      case OFTDateTime:
+	PROTECT(ans=allocVector(STRSXP,nRows));
+	break;
+      case OFTTime:
+	PROTECT(ans=allocVector(STRSXP,nRows));
+	break;
     default:
       error("unsupported field type: %s",poField->GetFieldTypeName(
         poField->GetType()));
@@ -232,6 +241,22 @@ extern "C" {
           SET_STRING_ELT(ans,iRow,mkChar(poFeature->GetFieldAsString(iField)));
 	else SET_STRING_ELT(ans, iRow, NA_STRING);
 	break;
+      case OFTDate:
+	if (poFeature->IsFieldSet(iField)) 
+          SET_STRING_ELT(ans,iRow,mkChar(poFeature->GetFieldAsString(iField)));
+	else SET_STRING_ELT(ans, iRow, NA_STRING);
+	break;
+      case OFTDateTime:
+	if (poFeature->IsFieldSet(iField)) 
+          SET_STRING_ELT(ans,iRow,mkChar(poFeature->GetFieldAsString(iField)));
+	else SET_STRING_ELT(ans, iRow, NA_STRING);
+	break;
+      case OFTTime:
+	if (poFeature->IsFieldSet(iField)) 
+          SET_STRING_ELT(ans,iRow,mkChar(poFeature->GetFieldAsString(iField)));
+	else SET_STRING_ELT(ans, iRow, NA_STRING);
+	break;
+
       default:
         delete poFeature;
 	error("Unsupported field type. should have been caught before");
